<?php
/**
 * @package   FL Amp Plugin
 * @author    Дмитрий Васюков http://fictionlabs.ru
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

defined('_JEXEC') or die;

use Lullabot\AMP\AMP;
use Lullabot\AMP\Validate\Scope;

class plgSystemFLAMP extends JPlugin
{
	// Construct Function

	public function __construct($subject, $config)
	{
		parent::__construct($subject, $config);
		$this->loadLanguage();
 
        $this->app 		= JFactory::getApplication();
		$this->input 	= $this->app->input;
		$this->document = JFactory::getDocument();
		$this->session  = JFactory::getSession();
	}

	// On After Route Function

	public function onAfterRoute()
	{
		// Add Amp Tag
		if (!$this->app->isAdmin() && $this->input->get('format') != 'json' && !in_array($this->input->get('format'), array('raw', 'json', 'xml'))) {
			$this->document->addCustomTag('<link rel="amphtml" href="'.JURI::current().'?flview=amp" />');
		}
	}

	// On After Render Function

	public function onAfterRender()
	{

		if ($this->app->isAdmin() 
			|| ($this->input->get('flview') != 'amp' && $this->session->get('flview') != 'amp')
			|| in_array($this->input->get('option'), $this->params->get('pro_components', array()))
		) {
			return;
		}
		
		// Force redirect
		if ($this->input->get('flview') == 'noamp') {
			$this->session->set('flview', '');
			$this->app->redirect(Juri::current());
		}

		if ($this->input->get('flview') == 'amp' || $this->session->get('flview') == 'amp') {
			if ($this->session->get('flview') != 'amp') {
				$this->session->set('flview', 'amp');
			}	

			if ($this->input->get('tmpl') != 'component') {
				$this->app->redirect(Juri::current().'?tmpl=component&flview=amp');
			}			
		}

		$body 			= $this->app->getBody();

		preg_match("/<body[^>]*>(.*?)<\/body>/is", $body, $matches);

		$bodyContent 	= $matches[1];

		// Check Cache
		if ($this->params->get('pro_cache_enable', 0)) {
			$cacheId 	= md5(serialize(JURI::current()));
			$cache 		= JFactory::getCache('plg_flamp', '');

			$cache->setCaching(true);
			$cache->setLifeTime($this->params->get('pro_cache_time', JFactory::getConfig()->get('cachetime', 1140)));

			$ampHtml 	= $cache->get($cacheId, 'plg_flamp');

			if (!$ampHtml) {
				$ampHtml = $this->getAmpHtml($bodyContent);

				$cache->store($ampHtml, $cacheId, 'plg_flamp');
			}
		} else {
			$ampHtml 	= $this->getAmpHtml($bodyContent);
		}

		$canonical 	= $this->getCanonical();
		$pageHtml 	= $this->params->get('pro_minify_enable') ? $this->minifyHtml($this->getPage($ampHtml, $canonical)) : $this->getPage($ampHtml, $canonical);

		$this->app->setBody($pageHtml);
	}

	// Get Amp Html

	public function getPage($body, $canonical)
	{
		$normalizeCss = $this->read(__DIR__.'/assets/css/normalize.css');
		$flAmpCss = $this->read(__DIR__.'/assets/css/flamp.css');

		// Get the path for the layout file
		$path = JPluginHelper::getLayoutPath('system', 'flamp');
		// Render Template
		ob_start();
		include $path;
		$html = ob_get_clean();

		return $html;
	}

	// Get Amp Html

	public function getAmpHtml($body)
	{
		require_once __DIR__ . '/libs/amp/vendor/autoload.php';

		$options = array();

		// Get Exclude Selectors
		$excludeSelectors = $this->params->get('pro_selectors');
		$imgLinks      	= $this->params->get('settings_img_links', 0);

		if ($excludeSelectors) {
			$excludeSelectors 			 = json_decode($this->params->get('pro_selectors'), true);
			$options['remove_selectors'] = $excludeSelectors['pro_selectors'];
		}

		if ($imgLinks == 1) {
			$options['amp_lightbox_image'] = true; // Add Lightbox
		} else if ($imgLinks == 2) {
			$options['amp_unwrap_image'] = true; // Unwrap Img From Link To Img
		}

		// New Amp
		$amp = new AMP();

		$amp->loadHtml($body, $options);

		$ampHtml = $amp->convertToAmpHtml();

		if ($this->params->get('pro_debug_enable', 0)) {
			$ampHtml .= '<div class="amp-debug"><pre>'.$amp->warningsHumanHtml().'</pre></div>';
		}

		return $ampHtml;
	}

	// Get File Content

	public function read($path)
	{
        $path = JPath::clean($path);

        if (JFile::exists($path)) {
			$contents = file_get_contents($path);

            return str_replace('!important', '', $contents);
        }

        return null;
    }

    // Convert Hex To RGBA Function
    public function hex2rgba($color, $opacity = false)
    { 
		$default = 'rgb(0,0,0)';
	 
		//Return default if no color provided
		if(empty($color)) {
			return $default; 
		}
	 
		//Sanitize $color if "#" is provided 
        if ($color[0] == '#' ) {
        	$color = substr( $color, 1 );
        }
 
        //Check if color has 6 or 3 characters and get values
        if (strlen($color) == 6) {
                $hex = array( $color[0] . $color[1], $color[2] . $color[3], $color[4] . $color[5] );
        } elseif ( strlen( $color ) == 3 ) {
                $hex = array( $color[0] . $color[0], $color[1] . $color[1], $color[2] . $color[2] );
        } else {
                return $default;
        }
 
        //Convert hexadec to rgb
        $rgb =  array_map('hexdec', $hex);
 
        //Check if opacity is set(rgba or rgb)
        if($opacity){
        	if(abs($opacity) > 1)
        		$opacity = 1.0;
        	$output = 'rgba('.implode(",",$rgb).','.$opacity.')';
        } else {
        	$output = 'rgb('.implode(",",$rgb).')';
        }
 
        //Return rgb(a) color string
        return $output;
	}

	// Inverse HEX Color Function

	public function hexInvert($color)
	{
	    $color = str_replace('#', '', $color);

	    if (strlen($color) != 6) {
	    	return '#000000'; 
	    }

	    $rgb = '';

	    for ($x=0; $x < 3; $x++){
	        $c = 255 - hexdec(substr($color, (2*$x), 2));
	        $c = ($c < 0) ? 0 : dechex($c);
	        $rgb .= (strlen($c) < 2) ? '0'.$c : $c;
	    }

	    return '#'.$rgb;
	}

	// Render Module By Position

	public function renderModule($position, $options = array())
	{
		jimport('joomla.application.module.helper');

        $document    = JFactory::getDocument();
        $renderer    = $document->loadRenderer('modules');
        $moduleHtml	 = $renderer->render($position, $options, null);

        return $moduleHtml;
	}

	// Get Images From String

	public function getImageFromText($text)
	{
		preg_match('/< *amp-img[^>]*src *= *["\']?([^"\']*)/i', $text, $matches);

		if (!empty($matches)) {
			return $matches[1];
		}

		return null;
	}

	// Check Image Url

	public function getAbsoluteUrl($url)
	{
		if (strpos($url, 'http://') === 0 || strpos($url, 'https://') === 0) { // Check External Url
        	return $url;
        } else {
        	return JUri::root().trim($url, '\/');
        }
	}

    // Get Canonical Link

    public function getCanonical()
    {
    	$links 		= $this->document->_links;
    	$canonical 	= JURI::current();

    	if (!empty($links)) {
    		foreach ($links as $key => $link) { 
		     	if ($link['relation'] == 'canonical') { 
		     		$canonical = $this->getAbsoluteUrl($key);
		     	}
		    }
    	}

    	return $canonical;
    	
    }

    // Minify Html

    public function minifyHtml($html)
    {
	    $search = array(
	        '/\>[^\S ]+/s',     	// strip whitespaces after tags, except space
	        '/[^\S ]+\</s',     	// strip whitespaces before tags, except space
	        '/(\s)+/s',         	// shorten multiple whitespace sequences
	        '/<!--(.|\s)*?-->/',	// Remove HTML comments
	        '/\/\*(.|\s)*?\*\//',
	        "/\r\n/",
	        "/\r/",
	        "/\n/",
	        "/\t/",
	        '/  /',
	        '/    /',
	        '/    /'
	    );

	    $replace = array(
	        '>',
	        '<',
	        '\\1',
	        '',
	        '',
	        '',
	        '',
	        '',
	        '',
	        '',
	        '',
	        ''
	    );

	    $html = preg_replace($search, $replace, $html);

	    return $html;
	}
}